#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# Program Name:  CountsZipf.awk                                             #
# Author:        Brigitte Bigi                                              #
# Date:          Nov, 2005                                                  #
# Licence:       GPL                                                        #
# Brief:         Estimates the Zipf law from a sorted count file            #
# Usage:         Counts2Zipf.awk -h                                         #
#                                                                           #
# ######################################################################### #
# List of global variables:                                                 #
#    Output      output file name                                           #
#    TAB_Counts  counts of the unigrams                                     #
#    TAB_Ind     index of the unigrams                                      #
#    Sum         total number of words                                      #
# ######################################################################### #


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Zipf                                                   #
#                                                                           #
# Description: Estimates the Zipf constant                                  #
#                                                                           #
# ######################################################################### #
function FCT_Zipf ( w )
{
	return ( TAB_Ind[w] * ( TAB_Counts[w] / Sum))
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description: Print the usage and exit the program                         #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
	printf "Brief: Estimates the Zipf law from a sorted count file\n"
	printf "Output: a file with 4 columns [word,count,index,zipf]\n"
	printf "Usage: cat file.counts | Counts2Zipf.awk [options ...] [-h] \n\n"
	printf " where options include:\n"
	printf "  -o filename\twrite result to this file [default: stdout]\n"
	printf "  -s value\t the column number to sort the output (range 2..4)\n"
	printf "  -h	     	print this help\n"
	exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description: Gets arguments from the command line                         #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
	for (i=1; i<argc; i++)
	{
		if (argv[i] == "-h")
			FCT_Usage()
		else
		if (argv[i] == "-o")
		{
			Output = argv[i+1]
			i++
		}
		else
		if (argv[i] == "-s")
		{
			if (argv[i+1] > 4)
			{
				sortval=0
			}
			else
			{ 
				sortval = argv[i+1]
			}
			i++
		}
		else
		if (argv[i] ~ /^\-/)
		{
			printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
			FCT_Usage()
		}
		else
			break
	}
}


# ############################# BEGIN MAIN ################################ #


BEGIN {
	sortval = 0			# column so sort the result
	Output = "/dev/stdout"		# output count file name
	FCT_CommandLine(ARGC, ARGV)	# extract parameters
	delete ARGV
	ARGC=1
	Sum = 0				# total number of words
	Ind = 0				# index of each word
	c   = 0				# previous count
}

{
	# Insert only the 1-grams of the count file
	# NF=2 means 1->word and 2->count
	if (NF == 2)
	{
		# The count file must be sorted... verify!
		if ( c != 0 && $2 > c )
		{
			printf "ERROR: entry count file must be sorted\n" > "/dev/stderr"
			printf "%d is not superior or equal to %d\n", c, $2
			exit
		}

		# The index value changes only if the count has changed
		if ( c != $2 )
		{ 
			Ind ++
			c = $2
		}
		
		# Insert values in TAB
		TAB_Counts[$1] = $2 
		TAB_Ind[$1]    = Ind
		Sum += $2
	}
}

END{
	# Set the sort command (or not...)
	if (sortval < 2)
		sort = " > Output"
	else
	if (sortval == 3)
		sort = sprintf("sort -k %d -n -o %s", sortval, Output)
	else
		sort = sprintf("sort -k %d -nr -o %s", sortval, Output)

	# Print the result with or without sort
	for (w in TAB_Counts)
		print w,TAB_Counts[w],TAB_Ind[w],FCT_Zipf(w) | sort 

	close(sort)
}

# ############################## END MAIN ################################# #


