#! /usr/bin/awk -f
# ######################################################################### #
#                                                                           #
# File:     XML2Sent.awk                                                    #
# Authors:  Brigitte Bigi                                                   #
# Date:     Dec, 2005                                                       #
# Licence:  GPL                                                             #
# Brief:    Select sentences of a xml corpus                                #
# Help:     XML2Sent.awk -h                                                 #
#                                                                           #
# ######################################################################### #
#                                                                           #
# List of functions:                                                        #
#    FCT_LoadVocab                                                          #
#    FCT_IsOOV                                                              #
#    FCT_Usage                                                              #
#    FCT_CommandLine                                                        #
#    FCT_Print                                                              #
#                                                                           #
# List of global variables:                                                 #
#    Output      output file name                                           #
#    TAB_Voc     words of the vocabulary                                    #
#    vocabsize   number of words in the vocabulary                          #
#                                                                           #
# ######################################################################### #


# ######################################################################### #
#                                                                           #
#  Function FCT_LoadVocab                                                   #
#                                                                           #
#  Description: Load a vocabulary from a file                               #
#                                                                           #
#  Parameters:                                                              #
#       file is the file name of the vocabulary                             #
#                                                                           #
# ######################################################################### #
function FCT_LoadVocab (file)
{
        _ind = 0
        while ( getline < file )
        {
                TAB_Voc[_ind]=$0
                _ind++
        }
        return _ind
}


# ######################################################################### #
#                                                                           #
#  Function FCT_IsOOV                                                       #
#                                                                           #
#  Description: Dichotomic research for a word in TAB_Vocab                 #
#                                                                           #
#  Parameters:                                                              #
#     word is a string finded in TAB_Vocab (of vocabsize words)             #
#                                                                           #
# ######################################################################### #
function FCT_IsOOV (word)
{
        left   = 0
        right  = vocabsize+1

        while ( left != right )
        {
                median = int((right+left)/2)
                if ( TAB_Voc[median] > word )
                        right = median
                else
                {
                        if ( TAB_Voc[median] < word )
                                left = median+1
                        else
                        {
                                return 0
                        }
                }
        }
        return 1
}



# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: XML2Sent.awk [options]\n\n"
        printf "Where options include:\n"
        printf "  -o filename  write the output to this file (default: stdout)\n"
	printf "  -V filename  read vocabulary from this file\n"
	printf "  -u value     value > 0 and < 1 for the threshold of UNK\n"
	printf "  -M value     minimum number of elements for a sentence\n"
	printf "  -l filename  log file for rejected sentences\n"
	printf "  -n           print <s> null </s> if the sentence is empty\n"
        printf "  -h           print this help\n"
        printf "\n"
        printf " input text:  stdin\n"
        printf " output text: stdout (except if -o)\n"
        exit
}

# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
                if (argv[i] == "-o")
                {
                        Output = argv[i+1]
                        i++
                }
                else
                if (argv[i] == "-u")
                {
                        unkmax = argv[i+1]
                        i++
                }
                else
                if (argv[i] == "-M")
                {
                        wdmin = argv[i+1]
                        i++
                }
                else
                if (argv[i] == "-l")
                {
                        logfile = argv[i+1]
                        i++
                }
                else
                if (argv[i] == "-V")
                {
                        vocabsize = FCT_LoadVocab(argv[i+1])
                        i++
                }
                else
                if (argv[i] == "-n")
                {
                        nullsent = 1
                }
                else
                if (argv[i] ~ /^\-/)
                {
                        printf "invalid option -- %s\n", ARGV[i] > "/dev/stderr"
                        FCT_Usage()
                }
		else
                        break
        }
#        delete argv
#        argc=1
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Print                                                  #
#                                                                           #
# Description:                                                              #
#       Print TAB_ELT in range 1..wd                                        #
#                                                                           #
# ######################################################################### #
function FCT_Print ( file )
{
	printf "<s>" > file
	for (i=1; i<=wd; i++)
		printf " %s", TAB_ELT[i] > file
	print " </s>" > file
}


# ############################# BEGIN MAIN ################################ #

BEGIN {
	logfile     = "/dev/null"
        Output      = "/dev/stdout"     # output file name
	unkmax      = 1			# maximum unknown elements rate by sentence
	wdmin       = 0			# minimum number of elements by sentence
	nullsent    = 0			# boolean: print a null sentence if no <wd> were founded
        FCT_CommandLine(ARGC, ARGV)     # extract parameters
}

# Find a new sentence
/<sent[ >]/{
	wd = 0
}

# End of a sentence
/<\/sent>/{
	if (wd > 0)
	{
		rate = 0
		# Si la selection unk est activee
		if (unkmax < 1)
		{
			# Calcule le taux d'elements inconnus
			unk = 0
			for (i=1; i<=wd; i++)
				unk += FCT_IsOOV(TAB_ELT[i])
			rate = unk/wd
		}
		if (rate < unkmax  &&  wd >= wdmin )
			FCT_Print( Output )
		else
			FCT_Print( logfile )
	}
	else
	{
		if (nullsent == 1) printf "<s> null </s>\n" > Output
	}
}

# Find a new word...
/<str>/{
        # Get the word 
        s = index($0, ">")
        w = substr($0, s+1, length($0))
        e = index(w, "<")
        w = substr($0, s+1, e-1)
        gsub(/ /, "", w)

	# Put the word in tab
	wd++
	TAB_ELT[wd] = w
}

################################ END MAIN ###################################


